@@ -19,7 +19,6 @@ def _central_diff_task(atoms, i, j, delta, model, num_layers):
     atoms_b.positions[i, j] -= delta
     desc_b = model.get_descriptors(atoms_b, num_layers=num_layers)
 
-    # 중앙차분 결과: shape = (n_atoms, descriptor_dim)
     grad_ij = (desc_f - desc_b) / (2 * delta)
     return i, j, grad_ij
 
@@ -182,9 +181,9 @@ class GaussianProcess(object):
                     from mace_descriptor.calculators import mace_mp
                 except ImportError:
                     raise ImportError(
-                        "The 'mace' and 'mace-descriptor' package is required for using pre-trained MACE descriptors.\n"
+                        "The 'joblib', 'mace' and 'mace-descriptor' packages are required for using pre-trained MACE descriptors.\n"
                         "Please install it by running:\n\n"
-                        "    pip install mace-torch\n"
+                        "    pip install joblib\n"
                         "    pip install mace-torch\n"
                         "    pip install mace-descriptor\n\n"
                         "Note: This is a lightweight fork of the original MACE (mace-torch) package, "
@@ -199,8 +198,10 @@ class GaussianProcess(object):
                     from mace_descriptor.calculators import mace_off
                 except ImportError:
                     raise ImportError(
-                        "The 'mace-descriptor' package is required for using pre-trained MACE descriptors.\n"
+                        "The 'joblib', 'mace' and 'mace-descriptor' packages are required for using pre-trained MACE descriptors.\n"
                         "Please install it by running:\n\n"
+                        "    pip install joblib\n"
+                        "    pip install mace-torch\n"
                         "    pip install mace-descriptor\n\n"
                         "Note: This is a lightweight fork of the original MACE (mace-torch) package, "
                         "designed exclusively for descriptor extraction."
@@ -417,31 +418,40 @@ class GaussianProcess(object):
 
         if not get_variance:
             for i in range(0, eval_x_N_batch):
-                eval_fp, eval_dfp_dr = self.generate_descriptor(eval_images[eval_x_indexes[i][0]:eval_x_indexes[i][1]])
-                pred, kernel = self.eval_data_batch(eval_fp, eval_dfp_dr)
-
                 data_per_batch = eval_x_indexes[i][1] - eval_x_indexes[i][0]
+                eval_fp, eval_dfp_dr = self.generate_descriptor(eval_images[eval_x_indexes[i][0]:eval_x_indexes[i][1]])
+
+                pred, kernel = self.eval_data_batch(eval_fp, eval_dfp_dr)
                 E_hat[eval_x_indexes[i][0]:eval_x_indexes[i][1]] = pred[0:data_per_batch]
                 F_hat[eval_x_indexes[i][0]:eval_x_indexes[i][1], :] = apply_force_mask(F=pred[data_per_batch:].view(data_per_batch, -1),
                                                                                        atoms_mask=self.atoms_mask)
 
-                return E_hat, F_hat.view((Ntest, self.Natom, 3)), None
+            return E_hat, F_hat.view((Ntest, self.Natom, 3)), None, None
 
         else:
-            uncertainty = torch.empty((Ntest,), dtype=self.torch_data_type, device=self.device)
+            unc_e = torch.empty((Ntest,), dtype=self.torch_data_type, device=self.device)
+            unc_f = torch.zeros((Ntest, self.Natom * 3), dtype=self.torch_data_type, device=self.device)
+
             for i in range(0, eval_x_N_batch):
+                data_per_batch = eval_x_indexes[i][1] - eval_x_indexes[i][0]
                 eval_fp, eval_dfp_dr = self.generate_descriptor(eval_images[eval_x_indexes[i][0]:eval_x_indexes[i][1]])
+
                 pred, kernel = self.eval_data_batch(eval_fp, eval_dfp_dr)
+                E_hat[eval_x_indexes[i][0]:eval_x_indexes[i][1]] = pred[0:data_per_batch]
+                F_hat[eval_x_indexes[i][0]:eval_x_indexes[i][1], :] = apply_force_mask(F=pred[data_per_batch:].view(data_per_batch, -1),
+                                                                                       atoms_mask=self.atoms_mask)
 
                 var = self.eval_variance_batch(get_variance=get_variance,
                                                eval_fp=eval_fp,
                                                eval_dfp_dr=eval_dfp_dr,
                                                k=kernel)
+                std = torch.sqrt(torch.diagonal(var))
 
-                data_per_batch = eval_x_indexes[i][1] - eval_x_indexes[i][0]
-                uncertainty[eval_x_indexes[i][0]:eval_x_indexes[i][1]] = torch.sqrt(torch.diagonal(var)[0:data_per_batch]) / self.weight
+                unc_e[eval_x_indexes[i][0]:eval_x_indexes[i][1]] = std[0:data_per_batch] / self.weight
+                unc_f[eval_x_indexes[i][0]:eval_x_indexes[i][1], :] = apply_force_mask(F=std[data_per_batch:].view(data_per_batch, -1),
+                                                                                       atoms_mask=self.atoms_mask)
 
-                return E_hat, F_hat.view((Ntest, self.Natom, 3)), uncertainty
+            return E_hat, F_hat.view((Ntest, self.Natom, 3)), unc_e, unc_f.view((Ntest, self.Natom, 3))
 
     def eval_data_batch(self, eval_fp, eval_dfp_dr):
         # kernel between test point x and training points X
@@ -466,6 +476,7 @@ class GaussianProcess(object):
         """
 
         if get_variance:
+            # Kx=k -> x = K^(-1)k
             covariance = torch.matmul(k, torch.cholesky_solve(k.T.clone(), self.K_XX_L, upper=False))
 
             # Adjust variance by subtracting covariance
@@ -481,8 +492,7 @@ class GaussianProcess(object):
 
         pred, kernel = self.eval_data_per_data(eval_fp_i, eval_dfp_dr_i)
         E_hat = pred[0]
-        F_hat = apply_force_mask(F=pred[1:].view(1, -1),
-                                 atoms_mask=self.atoms_mask)
+        F_hat = apply_force_mask(F=pred[1:].view(1, -1), atoms_mask=self.atoms_mask)
 
         if not get_variance:
             return E_hat, F_hat.view((self.Natom, 3)), None
@@ -492,9 +502,12 @@ class GaussianProcess(object):
                                               eval_fp_i=eval_fp_i,
                                               eval_dfp_dr_i=eval_dfp_dr_i,
                                               k=kernel)
-            uncertainty = torch.sqrt(var[0, 0]) / self.weight
 
-            return E_hat, F_hat.view((self.Natom, 3)), uncertainty
+            std = torch.sqrt(torch.diagonal(var))
+            unc_e = std[0] / self.weight
+            unc_f = apply_force_mask(F=std[1:].view(1, -1), atoms_mask=self.atoms_mask)
+
+            return E_hat, F_hat.view((self.Natom, 3)), unc_e, unc_f.view((self.Natom, 3))
 
     def eval_data_per_data(self, eval_fp_i, eval_dfp_dr_i):
         # kernel between test point x and training points X
