@@ -2,10 +2,10 @@ from .reference_data import ReferenceData
 # from .reference_data_internal import ReferenceDataInternal
 from .additional_data import AdditionalData
 # from .additional_data_internal import AdditionalDataInternal
-from .prepare_data import get_N_batch, get_batch_indexes_N_batch, standard_output, inverse_standard_output
+from .prepare_data import get_N_batch, get_batch_indexes_N_batch, standard_output, inverse_standard_output, read_xsf_image
 
 __all__ = ["reference_data", "additional_data",
            "reference_data_internal",  # "ReferenceDataInternal",
            "ReferenceData", "AdditionalData",
            "additional_data_internal",  # "AdditionalDataInternal",
-           "prepare_data", "get_N_batch", "get_batch_indexes_N_batch", "standard_output", "inverse_standard_output"]
+           "prepare_data", "get_N_batch", "get_batch_indexes_N_batch", "standard_output", "inverse_standard_output", "read_xsf_image"]
